#!@PERL@
#
# proc-xferlog.pl,v 1.4 2004/03/21 05:14:53 kim Exp
#
# Summarize FTP transfers
#
# 1996-07-30  Kimmo Suominen
#
$totalbytes = 0;

while (<>) {
    chomp;
    @_ = split;
    $site{$_[11]}{$_[6]}++;		# count for each site
    $time{$_[11]}{$_[6]} = 0 unless defined($time{$_[11]}{$_[6]});
    $time{$_[11]}{$_[6]} += $_[5];	# transfer times
    $size{$_[11]}{$_[6]} = 0 unless defined($size{$_[11]}{$_[6]});
    $size{$_[11]}{$_[6]} += $_[7];	# transfer bytes
    $file{$_[11]}{$_[8]}++;		# count for each file
    $type{$_[11]}{$_[9]}++;		# count for each transfer type
    $conv{$_[11]}{$_[10]}++;		# count for each conversion
    $anon{$_[11]}{$_[12]}++;		# count for connection type
    $totalbytes += $_[7];		# total transferred bytes
}

$limit = 10;

printf " Total  %d kB\n\n", $totalbytes / 1024;

foreach $i (sort (keys %type)) {
    printf " Files  %s Connection Types\n", &typename($i);
    printf " -----  -------------------------------------------------------\n";
    &conncnt($type{$i}, *xfertype);
    &conncnt($conv{$i}, *convtype);
    &conncnt($anon{$i}, *logintype);
    printf "\n";
}

foreach $i (sort (keys %file)) {
    $title = "TOP $limit " . &typename($i) . " Files";
    &topfiles($title, $limit, $file{$i});
}

foreach $i (sort (keys %site)) {
    $title = "TOP $limit " . &typename($i) . " Sites";
    &topsites($title, $site{$i}, $time{$i}, $size{$i});
}

exit 0;

sub typename {
    local($type) = shift(@_);

    if ($type eq "i") {
	return "Incoming";
    }
    if ($type eq "o") {
	return "Outgoing";
    }
    return "in/out???";
}

sub xfertype {
    local($type) = shift(@_);

    if ($type eq "a") {
	return "ascii";
    }
    if ($type eq "b") {
	return "binary";
    }
    return "untyped";
}

sub convtype {
    local($type) = shift(@_);

    if ($type eq "C") {
	return "compressed";
    }
    if ($type eq "U") {
	return "uncompressed";
    }
    if ($type eq "T") {
	return "tarred";
    }
    if ($type eq "_") {
	return "unconverted";
    }
    return "converted(?)";
}

sub logintype {
    local($type) = shift(@_);

    if ($type eq "a") {
	return "anonymous access";
    }
    if ($type eq "g") {
	return "guest access";
    }
    if ($type eq "r") {
	return "real user";
    }
    return "unknown access";
}

sub conncnt {
    local(*arry) = shift(@_);
    local(*conv) = shift(@_);

    local($i);

    foreach $i (sort {$arry{$b} <=> $arry{$a} || $a cmp $b} (keys %arry)) {
	printf "%6d  %s transfers\n", $arry{$i}, &{*conv}($i);
    }
}

sub pathxlate {
    local($_) = shift(@_);
    s,/net/[^/]+/([^-/]+)-[0-9]+/,/\1/,;
    s,/src/NetBSD/cvsroot/doc/,/usr/doc/,;
    s,/src/NetBSD/cvsroot/htdocs/,/usr/htdocs/,;
    s,/src/NetBSD/cvsroot/pkgsrc/,/usr/pkgsrc/,;
    s,/src/NetBSD/cvsroot/src/,/usr/src/,;
    s,/src/NetBSD/cvsroot/xsrc/,/usr/xsrc/,;
    s,/usr/local/pkg/,/usr/pkg/,;
    s,/proj/ftp/root/mirrors/,/proj/ftp/root/,;
    s,/proj/ftp/root/pub/,/pub/,;
    return $_;
}

sub topfiles {
    local($title) = shift(@_);
    local($limit) = shift(@_);
    local(*file) = shift(@_);

    local($total) = 0;
    local($lines) = 0;
    local($i);

    printf " Files  %s\n", $title;
    printf " -----  -------------------------------------------------------\n";
    foreach $i (sort {$file{$b} <=> $file{$a} || $a cmp $b} (keys %file)) {
	$total += $file{$i};
	printf "%6d  %-s\n", $file{$i}, pathxlate($i);
	last if (++$lines > $limit);
    }
    printf " -----  -------------------------------------------------------\n";
    printf "%6d  %-s\n\n", $total, "TOTAL";
}

sub topsites {
    local($title) = shift(@_);
    local(*site) = shift(@_);
    local(*time) = shift(@_);
    local(*size) = shift(@_);

    local($totf) = 0;
    local($tots) = 0;
    local($totb) = 0;
    local($lines) = 0;
    local($i);

    printf " Files       kB   kB/s  %s\n", $title;
    printf " -----  -------  -----  ---------------------------------------\n";
    foreach $i (sort {$size{$b} <=> $size{$a} || $a cmp $b} (keys %size)) {
	$totc++;
	# pretend it took at least 1 second
	$time{$i}++ if $time{$i} < 1;
	$totf += $site{$i};
	$tots += $time{$i};
	$totb += $size{$i};
	printf
	    "%6d  %7d  %5.1f  %-s\n",
	    $site{$i},
	    $size{$i} / 1024,
	    $size{$i} / (1024 * $time{$i}),
	    $i;
	last if (++$lines > $limit);
    }
    printf " -----  -------  -----  ---------------------------------------\n";
    printf
	"%6d  %7d  %5.1f  %-s\n",
	$totf / $totc,
	$totb / (1024 * $totc),
	$totb / (1024 * $tots),
	"AVERAGE";
    printf
	"%6d  %7d  %5s  %-s\n\n",
	$totf,
	$totb / 1024,
	"",
	"TOTAL";
}
